# TODO: implement a method of storing model states,
# For each version, cycle through the parameters and store a full yaml file and then
# compare the new yaml file with the previous version and detect changes and write them to a migration script.
# To upgrade, type `python sfsi_migrate <file-name> <old-version> <target-version>`
# The migrator then with run all migration scripts between the new and the old version.

